"use client";

import { useState, useEffect } from "react";
import { collection, addDoc, updateDoc, deleteDoc, doc, getDocs, query, where, getDoc } from "firebase/firestore";
import { db } from "@/lib/firebase";
import { Student } from "@/lib/types";
import { useActivity } from "./useActivity";

export const useStudents = (classId?: string) => {
  const [students, setStudents] = useState<Student[]>([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  const { addActivity } = useActivity();

  useEffect(() => {
    const fetchStudents = async () => {
      try {
        setLoading(true);
        setError(null);
        
        let studentsQuery;
        const studentsCollection = collection(db, "students");

        if (classId) {
          studentsQuery = query(studentsCollection, where("classIds", "array-contains", classId));
        } else {
          studentsQuery = query(studentsCollection);
        }

        const studentsSnapshot = await getDocs(studentsQuery);
        const studentsList: Student[] = [];

        for (const studentDoc of studentsSnapshot.docs) {
          const data = studentDoc.data() as Omit<Student, "id">;
          
          // Get class information for each classId
          const classPromises = data.classIds?.map(async (classId: string) => {
            const classDoc = await getDoc(doc(db, "classes", classId));
            if (classDoc.exists()) {
              return {
                id: classDoc.id,
                name: classDoc.data().name || '',
                level: classDoc.data().level || undefined 
              };
            }
            return null;
          }) || [];

          const classData = (await Promise.all(classPromises))
            .filter((cls): cls is { id: string; name: string, level: string } => cls !== null);
          
          studentsList.push({
            id: studentDoc.id,
            ...data,
            classes: classData.length > 0 ? classData : undefined
          });
        }

        setStudents(studentsList);
      } catch (err) {
        console.error("Error fetching students:", err);
        setError("Failed to fetch students");
      } finally {
        setLoading(false);
      }
    };

    fetchStudents();
  }, [classId]);

  const addStudent = async (studentData: Omit<Student, "id">) => {
    try {
      // Extract classes and remove from the data to be saved
      const { classes, ...studentToSave } = studentData;
      
      const docRef = await addDoc(collection(db, "students"), {
        ...studentToSave,
        classIds: studentData.classIds || []
      });

      const newStudent = { 
        id: docRef.id, 
        ...studentToSave,
        classIds: studentData.classIds || [],
        classes: studentData.classes || []
      };

      setStudents([...students, newStudent]);

      await addActivity({
        type: "student_enrollment",
        title: "New Student Enrolled",
        subject: `${studentData.firstName} ${studentData.lastName}`,
        details: `Enrolled in ${studentData.classes?.map(c => c.name).join(", ") || "Unassigned Class"}`,
        entityId: docRef.id,
        date: new Date().toISOString()
      });

      return newStudent;
    } catch (err) {
      console.error("Error adding student:", err);
      setError("Failed to add student");
      throw err;
    }
  };

  const updateStudent = async (id: string, data: Partial<Student>) => {
    try {
      const studentRef = doc(db, "students", id);
      const studentSnap = await getDoc(studentRef);
      const currentStudent = studentSnap.exists() ? studentSnap.data() as Student : null;

      // Extract classes and remove from the data to be saved
      const { classes, ...dataToSave } = data;
      
      await updateDoc(studentRef, {
        ...dataToSave,
        classIds: data.classIds || currentStudent?.classIds || []
      });

      if (data.status && currentStudent && data.status !== currentStudent.status) {
        await addActivity({
          type: "status_change",
          title: "Student Status Updated",
          subject: `${currentStudent.firstName} ${currentStudent.lastName}`,
          details: `Status changed from ${currentStudent.status} to ${data.status}`,
          entityId: id,
          previousStatus: currentStudent.status,
          date: new Date().toISOString(),
        });
      }

      setStudents(students.map(student => 
        student.id === id ? { ...student, ...data } : student
      ));

      return { id, ...data };
    } catch (err) {
      console.error("Error updating student:", err);
      setError("Failed to update student");
      throw err;
    }
  };

  const deleteStudent = async (id: string) => {
    try {
      await deleteDoc(doc(db, "students", id));
      setStudents(students.filter(student => student.id !== id));
    } catch (err) {
      console.error("Error deleting student:", err);
      setError("Failed to delete student");
      throw err;
    }
  };

  const getStudentsByClass = async (classId: string) => {
    try {
      const q = query(collection(db, "students"), where("classId", "==", classId));
      const querySnapshot = await getDocs(q);
      
      return querySnapshot.docs.map(doc => ({
        id: doc.id,
        ...doc.data()
      })) as Student[];
    } catch (err) {
      console.error("Error getting students by class:", err);
      throw err;
    }
  };

    const refreshStudents = async () => {
    try {
      setLoading(true);
      setError(null);
      
      let studentsQuery;
      const studentsCollection = collection(db, "students");

      if (classId) {
        studentsQuery = query(studentsCollection, where("classIds", "array-contains", classId));
      } else {
        studentsQuery = query(studentsCollection);
      }

      const studentsSnapshot = await getDocs(studentsQuery);
      const studentsList: Student[] = [];

      for (const studentDoc of studentsSnapshot.docs) {
        const data = studentDoc.data() as Omit<Student, "id">;
        
        // Get class information for each classId
        const classPromises = data.classIds?.map(async (classId: string) => {
          const classDoc = await getDoc(doc(db, "classes", classId));
          if (classDoc.exists()) {
            return {
              id: classDoc.id,
              name: classDoc.data().name || '',
              level: classDoc.data().level || undefined 
            };
          }
          return null;
        }) || [];

        const classData = (await Promise.all(classPromises))
          .filter((cls): cls is { id: string; name: string, level: string } => cls !== null);
        
        studentsList.push({
          id: studentDoc.id,
          ...data,
          classes: classData.length > 0 ? classData : undefined
        });
      }

      setStudents(studentsList);
    } catch (err) {
      console.error("Error refreshing students:", err);
      setError("Failed to refresh students");
    } finally {
      setLoading(false);
    }
  };

  return {
    students,
    loading,
    error,
    addStudent,
    updateStudent,
    deleteStudent,
    getStudentsByClass,
    refresh: refreshStudents,
  };
};

